@@ -23,22 +23,44 @@ OutUnits = {'Ids': 'uA',
             'NoC': 'A**2',
             }
 
-ScalarParams = ['Ids', 'GM', 'GMV', 'Irms', 'Vrms', 'NoA', 'NoB', 'NoC']
+########################################################################################################################
+## Define default scalar parameters
+########################################################################################################################
+ScalarParamsDC = ['Ids', 'GM', 'GMV']
+ScalarParamsAC = ['Irms', 'Vrms', 'NoA', 'NoB', 'NoC']
 VgsScalar = -0.1*pq.V
-ScalarQueries = {'CNP': {'Param': 'Ud0',
-                         'Units': 'mV'},
-                 'RdsCNP': {'Param': 'Rds',
-                            'Vgs': 0*pq.V,
-                            'Ud0Norm': True,
-                            'Units': 'kOhm'
-                            },
-                 'IdsCNP': {'Param': 'Ids',
-                            'Vgs': 0*pq.V,
-                            'Ud0Norm': True,
-                            'Units': 'uA'
-                            },
-                 }
-for par in ScalarParams:
+ScalarQueriesDC = {'CNP': {'Param': 'Ud0',
+                            'Units': 'mV'},
+                   'IgMax': {'Param': 'IgMax',
+                              'Units': 'nA'
+                              },
+                   'IdsV01': {'Param': 'Ids',
+                              'Vgs': 0.1*pq.V,
+                              'Ud0Norm': False,
+                              'Units': 'uA'
+                              },
+                   'RdsCNP': {'Param': 'Rds',
+                              'Vgs': 0*pq.V,
+                              'Ud0Norm': True,
+                              'Units': 'kOhm'
+                              },
+                   'IdsCNP': {'Param': 'Ids',
+                              'Vgs': 0*pq.V,
+                              'Ud0Norm': True,
+                              'Units': 'uA'
+                              } ,
+                   }
+
+for par in ScalarParamsDC:
+    d = {'Param': par,
+         'Vgs': VgsScalar,
+         'Ud0Norm': True}
+    if par in OutUnits:
+        d['Units'] = OutUnits[par]
+    ScalarQueriesDC[par+'01'] = d
+
+ScalarQueries = ScalarQueriesDC.copy()
+for par in ScalarParamsAC:
     d = {'Param': par,
          'Vgs': VgsScalar,
          'Ud0Norm': True}
@@ -46,11 +68,31 @@ for par in ScalarParams:
         d['Units'] = OutUnits[par]
     ScalarQueries[par+'01'] = d
 
+########################################################################################################################
+## Define default Array parameters
+########################################################################################################################
 Vgs = np.linspace(-0.1, 0.6, 100) * pq.V
 VgsNorm = np.linspace(-0.4, 0.4, 100) * pq.V
-ArrayParams = ['Ids', 'GM', 'GMV', 'Irms', 'Vrms', 'NoA', 'NoB', 'NoC']
-ArrayQueries = {}
-for par in ArrayParams:
+ArrayParamsDC = ['Ids', 'GM', 'GMV', 'Ig']
+ArrayParamsAC = ['Irms', 'Vrms', 'NoA', 'NoB', 'NoC']
+ArrayQueriesDC = {}
+for par in ArrayParamsDC:
+    d = {'Param': par,
+         'Vgs': Vgs,
+         'Ud0Norm': False}
+    if par in OutUnits:
+        d['Units'] = OutUnits[par]
+    ArrayQueriesDC[par] = d
+
+    d = {'Param': par,
+         'Vgs': VgsNorm,
+         'Ud0Norm': True}
+    if par in OutUnits:
+        d['Units'] = OutUnits[par]
+    ArrayQueriesDC[par+'Norm']=d
+
+ArrayQueries = ArrayQueriesDC.copy()
+for par in ArrayParamsAC:
     d = {'Param': par,
          'Vgs': Vgs,
          'Ud0Norm': False}
@@ -65,19 +107,31 @@ for par in ArrayParams:
         d['Units'] = OutUnits[par]
     ArrayQueries[par+'Norm']=d
 
+########################################################################################################################
+## Define default Array parameters
+########################################################################################################################
+
 ClassQueries = ScalarQueries.copy()
 ClassQueries.update(ArrayQueries)
-
 pdAttr = {'Vgs': Vgs,
           'VgsNorm': VgsNorm,
           'ScalarCols': list(ScalarQueries.keys()),
           'ArrayCols': list(ArrayQueries.keys()),
           }
 
+ClassQueriesDC = ScalarQueriesDC.copy()
+ClassQueriesDC.update(ArrayQueriesDC)
+pdAttrDC = {'Vgs': Vgs,
+            'VgsNorm': VgsNorm,
+            'ScalarCols': list(ScalarQueriesDC.keys()),
+            'ArrayCols': list(ArrayQueriesDC.keys()),
+            }
 
-def CalcElectricalParams(dbRaw, ClsQueries, dfAttr):   
+
+def CalcElectricalParams(dbRaw, ClsQueries, dfAttr, pErrors=False):
     PdSeries = []
     for index, row in dbRaw.iterrows():
+        print("Calculating {} of {}".format(index, dbRaw.shape[0]))
         char = row['CharCl']
         Vds = char.GetVds()
         for vds in Vds:
@@ -88,7 +142,8 @@ def CalcElectricalParams(dbRaw, ClsQueries, dfAttr):
                 try:
                     val = char.Get(**park)
                 except:
-                    print('Error', parn, char.Name)
+                    if pErrors:
+                        print('Error', parn, char.Name)
                     continue
                 if val is None:
                     continue
@@ -103,9 +158,11 @@ def CalcElectricalParams(dbRaw, ClsQueries, dfAttr):
     DataTypes = dbRaw.dtypes
     DataTypes['Vds'] = np.float
     for col in dfAttr['ScalarCols']:
-        DataTypes[col] = np.float
+        if col in dfDat.columns:
+            DataTypes[col] = np.float
     for col in dfAttr['ArrayCols']:
-        DataTypes[col] = object
+        if col in dfDat.columns:
+            DataTypes[col] = object
     dfDat = dfDat.astype(DataTypes)
     
     ColUnits = {}
@@ -138,7 +195,7 @@ def LoadPickleData(FileIn):
             GateDict = dat
             continue
         DCclass = DataCharDC(dat)
-        DictClDC[chn] = (DCclass, )
+        DictClDC[chn] = DCclass
 
         if DevACVals is None:
             continue
@@ -148,7 +205,7 @@ def LoadPickleData(FileIn):
         acDat.pop('Ids')
         acDat['Vgs'] = acDat.pop('VgsAC')
         acDat['Vds'] = acDat.pop('VdsAC')
-        DictClAC[chn] = (DataCharAC(acDat), )
+        DictClAC[chn] = DataCharAC(acDat)
     
     return DictClDC, DictClAC, GateDict
 
@@ -236,7 +293,7 @@ def Data2Pandas(DictData):
         for dd in dats:
             pdser = {}
             pdser['Name'] = tn
-            pdser['CharCl'] = dd
+            pdser['CharCl'] = DataCharAC(dd)
             pdser['IsOk'] = dd.IsOK
             pdser['ChName'] = dd.ChName
             pdser['Date'] = dd.GetDateTime()
@@ -274,3 +331,5 @@ def Data2Pandas(DictData):
     DataTypes['Date'] = 'datetime64[ns]'
     
     return dfRaw.astype(DataTypes)
+
+
